@@ -1,11 +1,19 @@
 ctStanMatricesList <- function(ctstanmodel){
   m <- list()
   m$base <- c("T0MEANS","LAMBDA","DRIFT","DIFFUSION","MANIFESTVAR","MANIFESTMEANS", "CINT","T0VAR","TDPREDEFFECT")
-  if('PARS' %in% ctstanmodel$pars$matrix) m$base <- c(m$base,'PARS')
-  m$dynamic <- c('DRIFT','DIFFUSION','CINT')
+
+  m$driftcintpars <- c('DRIFT','CINT')
+  m$diffusion <- 'DIFFUSION'
   m$tdpred <- 'TDPREDEFFECT'
   m$measurement <- c('LAMBDA','MANIFESTMEANS','MANIFESTVAR')
   m$t0 <- c('T0MEANS','T0VAR')
+  if('PARS' %in% ctstanmodel$pars$matrix) {
+    m$base <- c(m$base, 'PARS')
+    # m <- lapply(m, function(mi) { #if state reference made in any PARS param, include PARS in 
+    #   if(any(grepl('state[',ctstanmodel$pars$param[ctstanmodel$pars$matrix %in% 'PARS'],fixed=TRUE))) mi <- c(mi,'PARS')
+    #   return(mi)
+    # })
+  }
   return(m)
 }
 
@@ -29,11 +37,11 @@ ctStanCalcsList <- function(ctstanmodel){
   for(mati in unique(ctstanmodel$pars$matrix)){
     temp <- gsub(mati,paste0('s',mati),temp)
   }
-  
-  calcs <- lapply(mats[-1], function(mlist){
-    unlist(lapply(mlist, function(y) temp[grep(paste0('s',y),temp)] ))
+  calcs <- lapply(mats[-1], function(mlist) { #add custom calculations to correct list of matrices
+    out <- temp[unlist(sapply(temp, function(y) any(sapply(mlist[!mlist %in% 'PARS'], function(mli) grepl(mli,y)))))]
+    return(out)
   })
-  
+
   ctstanmodel$calcs <- calcs
   return(ctstanmodel)
 }
@@ -49,19 +57,24 @@ for(mlist in names(mats[-1])){
   if(any(unlist(lapply(ctstanmodel$calcs[[mlist]], function(m) grepl('sPARS',m))))) mats[[mlist]]=c(mats[[mlist]],'PARS')
 }
 
+
+    #adjust diffusion calcs to diffusionsqrt
+    ctstanmodel$calcs$diffusion <- gsub('sDIFFUSION','sDIFFUSIONsqrt',ctstanmodel$calcs$diffusion)
+    # intoverpopdynamiccalcs <- gsub('sDIFFUSION','sDIFFUSIONsqrt',intoverpopdynamiccalcs)
+
 #save calcs without intoverpop for param intitialisation
 nlcalcs <- ctstanmodel$calcs
 
     #intoverpop calcs setup
-    intoverpopdynamiccalcs <- paste0('
+    intoverpopdriftcintparscalcs <- paste0('
     if(intoverpop==1){ 
       for(ri in 1:size(matsetup)){ //for each row of matrix setup
-        if(matsetup[ ri,5] > 0 && ( statei == 0 || statei == nlatent + matsetup[ ri,5])){ // if individually varying
-          if(',paste0('matsetup[ri, 7] == ', which(mats$base %in% mats$dynamic), collapse = ' || '),'){ 
+        if(matsetup[ ri,5] > 0){ // && ( statei == 0 || statei == nlatent + matsetup[ ri,5])){ // if individually varying -- consider reimplementing extra check
+          if(',paste0('matsetup[ri, 7] == ', which(mats$base %in% c('DRIFT','CINT','PARS')), collapse = ' || '),'){ 
           real newval;
           newval = tform(state[nlatent + matsetup[ri,5] ], matsetup[ri,4], matvalues[ri,2], matvalues[ri,3], matvalues[ri,4], matvalues[ri,6] ); 
-          ',paste0('if(matsetup[ri, 7] == ', which(mats$base %in% mats$dynamic),') s',
-            mats$base[which(mats$base %in% mats$dynamic)],'[matsetup[ ri,1], matsetup[ri,2]] = newval;', collapse = ' \n          '),'
+          ',paste0('if(matsetup[ri, 7] == ', which(mats$base %in% c('DRIFT','CINT','PARS')),') s',
+            mats$base[which(mats$base %in% c('DRIFT','CINT','PARS'))],'[matsetup[ ri,1], matsetup[ri,2]] = newval;', collapse = ' \n          '),'
           }
         }
       }
@@ -71,7 +84,7 @@ nlcalcs <- ctstanmodel$calcs
     ctstanmodel$calcs[[matlisti]] <- paste0(ctstanmodel$calcs[[matlisti]],';
     if(intoverpop==1){ 
       for(ri in 1:size(matsetup)){ //for each row of matrix setup
-        if(matsetup[ ri,5] > 0 && ( statei == 0 || statei == nlatent + matsetup[ ri,5])){ // if individually varying
+        if(matsetup[ ri,5] > 0){ // // if individually varying
           if(',paste0('matsetup[ri, 7] == ', which(mats$base %in% mats[[matlisti]]), collapse = ' || '),'){ 
           real newval;
           newval = tform(state[nlatent + matsetup[ri,5] ], matsetup[ri,4], matvalues[ri,2], matvalues[ri,3], matvalues[ri,4], matvalues[ri,6] ); 
@@ -82,13 +95,8 @@ nlcalcs <- ctstanmodel$calcs
       }
     }')
     }
-
     #end intoverpop setup
     
-    
-    #adjust diffusion calcs to diffusionsqrt
-    ctstanmodel$calcs$dynamic <- gsub('sDIFFUSION','sDIFFUSIONsqrt',ctstanmodel$calcs$dynamic)
-    intoverpopdynamiccalcs <- gsub('sDIFFUSION','sDIFFUSIONsqrt',intoverpopdynamiccalcs)
 
 
 ukfilterfunc<-function(ppchecking){
@@ -146,9 +154,9 @@ ukfilterfunc<-function(ppchecking){
       if(nldynamics==0){
         state[1:nlatent] = sT0MEANS[,1];
         ',paste0(nlcalcs$t0,';',collapse=' '),'
-        eta = sT0MEANS[,1]; //prior for initial latent state
-        if(ntdpred > 0) eta += sTDPREDEFFECT * tdpreds[rowi];
-        etacov =  sT0VAR;
+        eta[1:nlatent] = sT0MEANS[,1]; //prior for initial latent state
+        if(ntdpred > 0) eta[1:nlatent] += sTDPREDEFFECT * tdpreds[rowi];
+        etacov[1:nlatent,1:nlatent] =  sT0VAR;
       }
 
     } //end T0 matrices
@@ -163,11 +171,11 @@ ukfilterfunc<-function(ppchecking){
         } else if(T0check[rowi-1] == 1 && dT[rowi-2] != dT[rowi]){
           dtchange = 1;
         } else if(T0check[rowi-1] == 0 && dT[rowi-1] != dT[rowi]) dtchange = 1;
-        ',if(length(nlcalcs$dynamic) > 0) paste0('dtchange =1; // because calcs provided
-        ',paste0(nlcalcs$dynamic,';\n',collapse=' '),';\n'),'
+        ',if(length(c(nlcalcs$driftcintpars,nlcalcs$diffusion)) > 0) paste0('dtchange =1; // because calcs provided
+        ',paste0(c(nlcalcs$driftcintpars,nlcalcs$diffusion),';\n',collapse=' '),';\n'),'
         
         if(dtchange==1 || (T0check[rowi-1]==1 && (si <= DRIFTsubindex[nsubjects] || si <= CINTsubindex[nsubjects]))){
-          discreteDRIFT = expm2(append_row(append_col(sDRIFT,sCINT),rep_matrix(0,1,nlatent+1)) * dT[rowi]);
+          discreteDRIFT = matrix_exp(append_row(append_col(sDRIFT,sCINT),rep_matrix(0,1,nlatent+1)) * dT[rowi]);
         }
     
         if(dtchange==1 || (T0check[rowi-1]==1 && (si <= DIFFUSIONsubindex[nsubjects]|| si <= DRIFTsubindex[nsubjects]))){
@@ -199,41 +207,44 @@ ukfilterfunc<-function(ppchecking){
       if(T0check[rowi]==0){
         matrix[nlatentpop,nlatentpop] J;
         vector[nlatent] base;
+        for(stepi in 1:integrationsteps[rowi]){
         J = rep_matrix(0,nlatentpop,nlatentpop); //dont necessarily need to loop over tdpreds here...
-        if(continuoustime==1){
-          matrix[nlatentpop,nlatentpop] Je;
-          matrix[nlatent*2,nlatent*2] dQi;
-          for(stepi in 1:integrationsteps[rowi]){
-            for(statei in 0:nlatentpop){
-              if(statei>0){
-                J[statei,statei] = 1e-6;
-                state = eta + J[,statei];
-              } else {
-                state = eta;
-              }
-              ',paste0(ctstanmodel$calcs$dynamic,';\n',collapse=' '),';\n', intoverpopdynamiccalcs,' //do we need intoverpop calcs here? and remove diffusion calcs and do elsewhere
-              if(statei== 0) {
-                base = sDRIFT * state[1:nlatent] + sCINT[,1];
-              }
-              if(statei > 0) {
-                J[1:nlatent,statei] = (( sDRIFT * state[1:nlatent] + sCINT[,1]) - base)/1e-6;
-              }
+          for(statei in 0:nlatentpop){
+            if(statei>0){
+              J[statei,statei] = 1e-6;
+              state = eta + J[,statei];
+            } else {
+              state = eta;
             }
-            ',paste0(ctstanmodel$calcs$dynamic,';\n',collapse=' '),' //find a way to remove this repeat
-            Je= expm2(J * dTsmall[rowi]) ;
-
-        //print("Je = ", Je,"  J = ", J, "  sDRIFT = ", sDRIFT, "  state  = ", state, "  sDIFFUSION = ", sDIFFUSION, "  sasymDIFFUSION = ", sasymDIFFUSION);
-            discreteDRIFT = expm2(append_row(append_col(sDRIFT,sCINT),rep_vector(0,nlatent+1)\') * dTsmall[rowi]);
+            ',paste0(ctstanmodel$calcs$driftcintpars,';\n',collapse=' '),';\n', intoverpopdriftcintparscalcs,' 
+            if(statei== 0) {
+              base = sDRIFT * state[1:nlatent] + sCINT[,1];
+            }
+            if(statei > 0) {
+              J[1:nlatent,statei] = (( sDRIFT * state[1:nlatent] + sCINT[,1]) - base)/1e-6;
+            }
+          }
+          ',paste0(ctstanmodel$calcs$diffusion,';\n',collapse=' '),' //not repeating other calcs because 1e-6 is small
+          if(continuoustime==1){
+            matrix[nlatentpop,nlatentpop] Je;
+            matrix[nlatent*2,nlatent*2] dQi;
+            Je= matrix_exp(J * dTsmall[rowi]) ;
+            discreteDRIFT = matrix_exp(append_row(append_col(sDRIFT,sCINT),rep_vector(0,nlatent+1)\') * dTsmall[rowi]);
             sasymDIFFUSION = to_matrix(  -kronsum(J[1:nlatent,1:nlatent]) \\ to_vector(tcrossprod(sDIFFUSIONsqrt)), nlatent,nlatent);
             discreteDIFFUSION =  sasymDIFFUSION - quad_form( sasymDIFFUSION, Je[1:nlatent,1:nlatent]\' );
             etacov = quad_form(etacov, Je\');
-            etacov[1:nlatent,1:nlatent] += discreteDIFFUSION;
+            etacov[1:nlatent,1:nlatent] += discreteDIFFUSION; //may need improving
             eta[1:nlatent] = (discreteDRIFT * append_row(eta[1:nlatent],1.0))[1:nlatent];
           }
+        if(continuoustime==0){ //test this
+          etacov = quad_form(etacov, J\');
+          etacov[1:nlatent,1:nlatent] += sDIFFUSION; //may need improving re sDIFFUSION
+          discreteDRIFT=append_row(append_col(sDRIFT,sCINT),rep_matrix(0,1,nlatent+1));
+          discreteDRIFT[nlatent+1,nlatent+1] = 1;
+          eta[1:nlatent] = (discreteDRIFT * append_row(eta[1:nlatent],1.0))[1:nlatent];
+        }
         }
 
-        if(continuoustime==0){ //need covariance in here
-        }
       } // end of non t0 time update
   
   
@@ -356,22 +367,22 @@ if(verbose > 1) print("etaprior = ", eta, " etapriorcov = ",etacov);
 '    
 ,if(ppchecking) paste0('
 {
-int skipupd = 0;
-        for(vi in 1:nobs_y[rowi]){
-          if(fabs(ypred[o[vi]]) > 1e10 || is_nan(ypred[o[vi]]) || is_inf(ypred[o[vi]])) {
-            skipupd = 1; 
-            ypred[o[vi]] =99999;
-if(verbose > 1) print("pp ypred problem! row ", rowi);
-          }
-        }
-        if(skipupd==0){ 
+//int skipupd = 0;
+//        for(vi in 1:nobs_y[rowi]){
+//          if(fabs(ypred[o[vi]]) > 1e10 || is_nan(ypred[o[vi]]) || is_inf(ypred[o[vi]])) {
+//            skipupd = 1; 
+//            ypred[o[vi]] =99999;
+//if(verbose > 1) print("pp ypred problem! row ", rowi);
+//          }
+//        }
+//        if(skipupd==0){ 
           if(ncont_y[rowi] > 0) ypredcov_sqrt[o0,o0]=cholesky_decompose(makesym(ypredcov[o0, o0],verbose,1)); 
           if(ncont_y[rowi] > 0) Ygen[ rowi, o0] = ypred[o0] + ypredcov_sqrt[o0,o0] * Ygenbase[rowi,o0]; 
           if(nbinary_y[rowi] > 0) for(obsi in 1:size(o1)) Ygen[rowi, o1[obsi]] = ypred[o1[obsi]] > Ygenbase[rowi,o1[obsi]] ? 1 : 0; 
-          for(vi in 1:nobs_y[rowi]) if(is_nan(Ygen[rowi,o[vi]])) {
-            Ygen[rowi,o[vi]] = 99999;
-print("pp ygen problem! row ", rowi);
-          }
+//          for(vi in 1:nobs_y[rowi]) if(is_nan(Ygen[rowi,o[vi]])) {
+//            Ygen[rowi,o[vi]] = 99999;
+//print("pp ygen problem! row ", rowi);
+//          }
         if(nlmeasurement==0){ //linear measurement
           if(intoverstates==1) { //classic kalman
             for(wi in 1:nmanifest){ 
@@ -380,12 +391,12 @@ print("pp ygen problem! row ", rowi);
           }
         }
         err[o] = Ygen[rowi,o] - ypred[o]; // prediction error
-        }
+//        }
 if(verbose > 1) {
 print("rowi ",rowi, "  si ", si, 
           "  eta ",eta,"  etacov ",etacov,"  ypred ",ypred,"  ypredcov ",ypredcov, "  K ",K,
           "  sDRIFT ", sDRIFT, " sDIFFUSION ", sDIFFUSION, " sCINT ", sCINT, "  sMANIFESTVAR ", diagonal(sMANIFESTVAR), "  sMANIFESTMEANS ", sMANIFESTMEANS, 
-          "  sT0VAR", sT0VAR, " sT0MEANS ", sT0MEANS,
+          "  sT0VAR", sT0VAR, " sT0MEANS ", sT0MEANS, 
           "  rawpopsd ", rawpopsd, "  rawpopsdbase ", rawpopsdbase, "  rawpopmeans ", rawpopmeans );
         print("discreteDRIFT ",discreteDRIFT,  "  discreteDIFFUSION ", discreteDIFFUSION)
 }
@@ -616,8 +627,9 @@ functions{
       }
     }
   return z;
-  }
-
+ }
+  
+   
   matrix expm2(matrix M){
     matrix[rows(M),rows(M)] out;
     int z[rows(out)] = checkoffdiagzero(M);
